//const margin = { top: 20, right: 30, bottom: 50, left: 50 };

function resize() {
    const container = document.getElementById('time-series-container');
    const width = container.clientWidth - margin.left - margin.right;
    const height = container.clientHeight - margin.top - margin.bottom;
    return { width, height };
}

d3.csv("schengen_data.csv").then((data) => {
    console.log(data)
    const { width, height } = resize();
    const allStates = new Set(data.map(d => d["Schengen State"]));
    const color = d3.scaleOrdinal(d3.schemeCategory10).domain(allStates);

    const svg = d3.select("#time-series-container")
        .append("svg")
        .attr("width", width + margin.left + margin.right)
        .attr("height", height + margin.top + margin.bottom)
        .append("g")
        .attr("transform", `translate(${margin.left+margin.right},${margin.top+margin.bottom})`);

    function updateGraph(type) {
        const processedData = d3.rollups(
            data,
            v => d3.sum(v, d => +d[type] || 0),
            d => +d["Year"],
            d => d["Schengen State"]
        ).sort((a, b) => a[0] - b[0]); // Sort by year

        const aggregatedData = processedData.map(([year, states]) => {
            const yearObj = { year: +year };
            for (const [state, sum] of states) {
                yearObj[state] = sum;
            }
            return yearObj;
        });

        const x = d3.scaleLinear()
            .domain(d3.extent(aggregatedData, d => d.year))
            .range([0, width-margin.left-margin.right]);

        const y = d3.scaleLinear()
            .domain([0, d3.max(aggregatedData, d =>
                d3.max([...allStates].map(state => d[state] || 0))
            )])
            .nice()
            .range([height-margin.top-margin.bottom, 0]);

        svg.selectAll(".axis").remove();
        svg.selectAll(".line").remove();

        svg.append("g")
            .attr("transform", `translate(0,${height-margin.top-margin.bottom})`)
            .attr("class", "axis")
            .call(d3.axisBottom(x).tickFormat(d3.format("d")));

        svg.append("g")
            .attr("class", "axis")
            .call(d3.axisLeft(y));

        console.log(aggregatedData)

        for (const state of allStates) {
            const stateData = aggregatedData.map(d => ({ year: d.year, value: d[state] || 0 }));

            svg.append("path")
                .datum(stateData)
                .attr("class", "line")
                .attr("d", d3.line()
                    .x(d => x(d.year))
                    .y(d => y(d.value))
                )
                .attr("id", (d)=>console.log(d))
                .attr("stroke", color(state))
                .attr("stroke-width", 1.5)
                .attr("fill", "none")
                .on("mouseover", function() {
                    d3.select(this).classed("highlighted", true);
                })
                .on("mouseout", function() {
                    d3.select(this).classed("highlighted", false);
                });
        }

        const legend = svg
            .selectAll(".legend-group")
            .data([null])
            .enter().append("g").attr("class", "legend-group")
            .attr("transform", `translate(${width - 150}, 20)`)
            .selectAll(".legend-item")
            .data(Array.from(allStates))
            .enter()
            .append("g")
            .attr("class", "legend-item")
            .attr("transform", (d, i) => `translate(0, ${i * 20})`)
            .each(function(d, i) {
                const group = d3.select(this);

                group.append("rect")
                    .attr("width", 10)
                    .attr("height", 10)
                    .attr("fill", color(d))
                    .on("mouseover", () => {
                        svg.selectAll(".line").classed("highlighted", line => line === d);
                    })
                    .on("mouseout", () => {
                        svg.selectAll(".line").classed("highlighted", false);
                    });

                group.append("text")
                    .attr("x", 20)
                    .attr("y", 10)
                    .attr("text-anchor", "start")
                    .style("text-transform", "capitalize")
                    .text(d);
            });
    }

    document.getElementById("visa-type-selector").addEventListener("change", (event) => {
        const type = event.target.value;
        updateGraph(type);
    });

    // Initialize with "Total  uniform visas issued (including MEV)"
    updateGraph("Total  uniform visas issued (including MEV)");
});