const margin = { top: 20, right: 30, bottom: 50, left: 50 };
const container = document.getElementById('tree-container');
const width = container.offsetWidth-15//- margin.left - margin.right;
const height = container.offsetHeight-15//- margin.top - margin.bottom;

let currentData;
let hierarchyStack = ["Schengen States"];

d3.csv("schengen_data.csv").then((data) => {
    currentData = data;
    updateTree("Schengen States");
});

function updateTree(filterKey) {
    // Filter data based on the selected key
    let filteredData;

    if (filterKey === "Schengen States") {
        filteredData = d3.group(
            currentData,
            d => d["Schengen State"],
            d => d["Country where consulate is located"],
            d => d["Consulate"]
        );
    } else if (hierarchyStack.length === 2) {
        // Filter for countries in the selected state
        filteredData = d3.group(
            currentData.filter(d => d["Schengen State"] === filterKey),
            d => d["Country where consulate is located"],
            d => d["Consulate"]
        );
    } else if (hierarchyStack.length === 3) {
        // Filter for consulates in the selected country
        const selectedCountry = hierarchyStack[2];
        filteredData = d3.group(
            currentData.filter(d => d["Country where consulate is located"] === selectedCountry),
            d => d["Consulate"]
        );
    }

    // Build hierarchy
    const rootData = buildHierarchy(filteredData, filterKey);
    const root = d3.hierarchy(rootData);

    //Remove children that are not direct successors of current node
    if (root.children) {
        root.children.forEach(child => {
            if (child.children) {
                child._children = child.children;
                child.children = null;
            }
        });
    }

    // Tree layout
    const treeLayout = d3.tree().size([height-10, width-200]);
    treeLayout(root);

    // Clear previous visualization
    d3.select("#tree-container svg").remove();

    // Append SVG
    const svg = d3.select("#tree-container")
        .append("svg")
        .attr("width", "100%")
        .attr("height", "100%")
        .append("g")
        .attr("transform", `translate(${margin.left+30},${margin.top-10})`);

    // Links
    svg.selectAll(".link")
        .data(root.links())
        .enter()
        .append("path")
        .attr("class", "link")
        .attr("d", d3.linkHorizontal()
            .x(d => d.y)
            .y(d => d.x)
        );

    // Nodes
    const node = svg.selectAll(".node")
        .data(root.descendants())
        .enter()
        .append("g")
        .attr("class", "node")
        .attr("transform", d => `translate(${d.y},${d.x})`);

    node.append("circle")
        .style("cursor", hierarchyStack.length < 3 ? "pointer" : "")
        .attr("r", 5)
        .on("click", (event, d) => {
            if (hierarchyStack.length < 3 && d.depth === 1 || d.depth === 2) { // Go deeper for states or countries
                d3.select(".tooltip").remove();
                hierarchyStack.push(d.data.name);
                updateTree(d.data.name);
                updateBackButton();
            }
        })
        .on("mouseover", (event, d) => {
            const tooltip = d3.select("body").append("div")
                .attr("class", "tooltip")
                .style("left", `${event.pageX + 5}px`)
                .style("top", `${event.pageY - 5}px`)
                .text(d.data.name);
        })
        .on("mouseout", () => {
            d3.select(".tooltip").remove();
        });

    node.append("text")
        .attr("dy", 3)
        .attr("x", d => d.children ? -10 : 10)
        .style("text-anchor", d => d.children ? "end" : "start")
        .text(d => d.data.name);
}


function buildHierarchy(nest, name) {
    return {
        name,
        children: Array.from(nest, ([key, value]) => {
            if (!key) return null;
            return value instanceof Map
                ? buildHierarchy(value, key)
                : { name: `${key}: ${value.length}` };
        }).filter(d => d)
    };
}

function updateBackButton() {
    const backButton = document.getElementById("back-button");
    backButton.disabled = hierarchyStack.length <= 1;
}

document.getElementById("back-button").addEventListener("click", () => {
    if (hierarchyStack.length > 1) {
        hierarchyStack.pop();
        const previousKey = hierarchyStack[hierarchyStack.length - 1];
        updateTree(previousKey);
        updateBackButton();
    }
});